@@ -38,30 +38,30 @@ class Gui_CLI
 		@ripping_log = ""
 		@ripping_progress = 0.0
 		@encoding_progress = 0.0
-		@finished = false
-		@settings = $rr_defaultSettings
-		if File.exist?(File.expand_path(@options.file))
-			load_configfile()
-			if @options.configure; edit_settings() end
+		@settingsClass = Settings.new(@options.file)
+		@settings = @settingsClass.settings
+		
+		if !@settingsClass.configFound || @options.configure
+			edit_settings()
 		else
-			edit_settings() # Always set up settings if no config file exists.
-		end		
-		get_cd_info()
+			get_cd_info()
+		end
 	end
 
 	def parse_options
 		@options = OpenStruct.new
-		@options.file = "~/.rubyripper/settings"
+		@options.file = false
+		@options.version = false
 		@options.help = false
 		@options.verbose = false
 		@options.configure = false
-		@options.all = false
+		@options.defaults = false
 		@options.help = false
 
 		opts = OptionParser.new(banner = nil, width = 20, indent = ' ' * 2) do |opts|
 			opts.on("-V", "--version", _("Show current version of rubyripper.")) do
 				puts "Rubyripper version #{$rr_version}."
-				exit()
+				@options.version = true
 			end
 			opts.on("-f", "--file <FILE>", _("Load configuration settings from file <FILE>.")) do |f|
 				@options.file = f
@@ -72,8 +72,8 @@ class Gui_CLI
 			opts.on("-c", "--configure", _("Change configuration settings.")) do |c|
 				@options.configure = c
 			end
-			opts.on("-a", "--all", _("Rip all tracks. Skip any questions about track selection.")) do |a|
-				@options.all = a
+			opts.on("-d", "--defaults", _("Skip questions and rip the disc.")) do |d|
+				@options.defaults = true
 			end
 			opts.on_tail("-h", "--help", _("Show this usage statement.")) do |h|
 				puts opts
@@ -88,11 +88,11 @@ class Gui_CLI
 			exit
 		end
 
-		if @options.help; exit end
+		if @options.help || @options.version; exit end
 
 		puts _("Verbose output specified.") if @options.verbose
 		puts _("Configure option specified.") if @options.configure
-		puts _("Rip all tracks specified.") if @options.all
+		puts _("Skip questions and rip the disc.") if @options.defaults
 		puts _("Use config file ") + @options.file if @options.file
 	end
 	
@@ -164,8 +164,12 @@ class Gui_CLI
 			puts ""
 			show_settings()
 		end
-		save_configfile()
-		unless get_answer(_("Do you want to continue with the program now? (y/n) : "), "yes", _("y")) ; exit() end
+		@settingsClass.save(@settings)
+		if get_answer(_("Do you want to continue with the program now? (y/n) : "), "yes", _("y"))
+			get_cd_info()
+		else
+			exit()
+		end
 	end
 	
 	def set_codec(codec) # little help function for edit_settings
@@ -179,7 +183,7 @@ class Gui_CLI
 	end
 	
 	def set_cdrom # little help function for edit_settings
-		@settings['cdrom'] = get_answer(_("Cdrom device : "), "open", cdrom_drive())
+		@settings['cdrom'] = get_answer(_("Cdrom device : "), "open", @settings['cdrom'])
 		@settings['offset'] = get_answer(_("Offset for drive : "), "number", 0)
 	end
 
@@ -204,32 +208,6 @@ class Gui_CLI
 			end
 		end
 	end
-
-	def load_configfile # load config file
-		if File.exist?(File.expand_path(@options.file))
-			file = File.new(File.expand_path(@options.file),'r')
-			while line = file.gets
-				key, value = line.split('=', 2)
-				value.rstrip! # remove the trailing newline character
-				if value == "false" ; value = false # replace the string with a bool
-				elsif value == "true" ; value = true  # replace the string with a bool
-				elsif value == "''" ; value = '' #replace a string that contains two quotes with an empty string
-				elsif value.to_i > 0 || value == '0' ; value = value.to_i
-				end
-				@settings[key] = value
-			end
-			file.close()
-		end
-	end
-	
-	def save_configfile # save config file
-		if not File.directory?(dirname = File.join(ENV['HOME'], '.rubyripper')) ; Dir.mkdir(dirname) end
-		file = File.new(File.expand_path(@options.file), 'w')
-		@settings.each do |key, value|
-			file.puts "#{key}=#{value}" if $rr_defaultSettings.include?(key)
-		end
-		file.close()
-	end
 	
 	def get_answer(question, answer, default)
 		succes = false
@@ -248,22 +226,24 @@ class Gui_CLI
 				if input == '' ; return default else return input end
 			elsif answer == "number"
 				STDOUT.print(question + " [#{default}] ")
-				input = STDIN.gets.strip.to_i #convert the answer to an integer value, if input is a text it will be 0.
-				if input == 0 ; return default else return input end
+				input = STDIN.gets.strip
+				#convert the answer to an integer value, if input is a text it will be 0.
+				#make sure that the valid answer of 0 is respected though
+				if input.to_i == 0 && input != "0" ; return default else return input.to_i end
 			else puts _("We should never get here") ; puts _("answer = %s, question = %s (error)") % [answer, question]
  			end
 		end
 	end
 
 	def get_cd_info
-		@settings['cd'] = Disc.new(@settings['cdrom'], @settings['freedb'], self, @settings['verbose']) # Analyze the TOC of disc in drive
+		@settings['cd'] = Disc.new(@settings, self) # Analyze the TOC of disc in drive
 		if @settings['cd'].audiotracks != 0 # a disc is found
 			puts _("Audio-disc found, number of tracks : %s, total playlength : %s") % [@settings['cd'].audiotracks, @settings['cd'].playtime]
 			if @settings['freedb'] #freedb enabled?
 				puts _("Fetching freedb info...")
 				handleFreedb()
 			else
-				freedb_finished()
+				showFreedb()
 			end
 		else 
 			puts @settings['cd'].error
@@ -285,7 +265,10 @@ class Gui_CLI
 			showFreedb()
 		elsif status[0] == "choices"
 			chooseFreedb(status[1])
-		elsif status[0] == "networkDown" || status[0] == "noMatches" || status[0] == "unknownReturnCode" || status[0] == "NoAudioDisc"
+		elsif status[0] == "noMatches"
+			update("error", status[1]) # display the warning, but continue anyway
+			showFreedb()
+		elsif status[0] == "networkDown" || status[0] == "unknownReturnCode" || status[0] == "NoAudioDisc"
 			update("error", status[1])
 		else
 			puts "Unknown error with Metadata class."
@@ -294,24 +277,33 @@ class Gui_CLI
 
 	def chooseFreedb(choices)
 		puts _("Freedb reported multiple possibilities.")
-		choices.each_index{|index| puts "#{index + 1}) #{choices[index]}"}
-		choice = get_answer(_("Please type the number of the one you prefer? : "), "number", 1)
-		handleFreedb(choice - 1)
+		if @options.defaults == true
+			puts _("The first freedb option is automatically selected (no questions allowed)")
+			handleFreedb(0)
+		else
+			choices.each_index{|index| puts "#{index + 1}) #{choices[index]}"}
+			choice = get_answer(_("Please type the number of the one you prefer? : "), "number", 1)
+			handleFreedb(choice - 1)
+		end
 	end
 
 	def showFreedb()
 		puts ""
 		puts _("FREEDB INFO\n\n")
 		puts _("DISC INFO")
-		puts _("Artist:") + " #{@settings['cd'].md.artist}"
-		puts _("Album:") + " #{@settings['cd'].md.album}"
-		puts _("Genre:") + " #{@settings['cd'].md.genre}"
-		puts _("Year:") + " #{@settings['cd'].md.year}"
+		print _("Artist:") ; print " #{@settings['cd'].md.artist}\n"
+		print _("Album:") ; print " #{@settings['cd'].md.album}\n"
+		print _("Genre:") ; print " #{@settings['cd'].md.genre}\n"
+		print _("Year:") ; print " #{@settings['cd'].md.year}\n"
 		puts ""
 		puts _("TRACK INFO")
 		
 		showTracks()
-		showFreedbOptions()
+		if @options.defaults
+			prepareRip()
+		else
+			showFreedbOptions()
+		end
 	end
 
 	def showTracks()
@@ -417,7 +409,7 @@ class Gui_CLI
 		
 		status = @rubyripper.settingsOk
 		if status == true
-			start()
+			@rubyripper.startRip()
 		else
 			update(status[0], status[1])
 		end
@@ -432,20 +424,12 @@ class Gui_CLI
 		puts ""
 
 		answer = get_answer(_("Please enter the number of your choice: "), "number", 1)
-		if answer == 1; @rubyripper.postfixDir() ; start()
-		elsif answer == 2; @rubyripper.overwriteDir() ; start()
+		if answer == 1; @rubyripper.postfixDir() ; @rubyripper.startRip()
+		elsif answer == 2; @rubyripper.overwriteDir() ; @rubyripper.startRip()
 		else cancelRip()
 		end
 	end
 
-	def start
-		@rubyripper.startRip()
-	#	while @finished == false
-	#		sleep(1) #loop until the finished signal is given.
-			#This is not ideal, but apparently the program gets terminated otherwise.
-	#	end
-	end
-
 	def update(modus, value=false)
 		if modus == "ripping_progress"
 			progress = "%.3g" % (value * 100)
@@ -461,15 +445,15 @@ class Gui_CLI
 			if get_answer(_("Do you want to change your settings? (y/n) : "), "yes",_("y")) ; edit_settings() end
 		elsif modus == "dir_exists"
 			dir_exists()
-		elsif modus == "finished"
-			@finished = true
 		end
 	end
 
 	def tracklist # Fill @settings['tracksToRip']
 		@settings['tracksToRip'] = Array.new
 		@settings['cd'].audiotracks.times{|number| @settings['tracksToRip'] << number + 1} # Start with all tracks selected
-		if @options.all || get_answer(_("\nShould all tracks be ripped ? (y/n) "), "yes", _('y'))
+		if @settings['image']
+			@settings['tracksToRip'] = ["image"]
+		elsif @options.defaults || get_answer(_("\nShould all tracks be ripped ? (y/n) "), "yes", _('y'))
 			puts _("Tracks to rip are %s") % [@settings['tracksToRip'].join(" ")]
 		else
 			succes = false
