@@ -36,24 +36,26 @@ rescue LoadError
 end
 
 class Main #the main window
-attr_reader :change_display, :instances, :update
+attr_reader :instances
 
 	def initialize
 		@instances = {'Preferences' => false, 'GtkMetadata' => false, 'ShortMessage' => false, 'RipStatus' => false, 'DirExists' => false, 'MultipleFreedbHits' => false, 'Summary' => false}
 		@current_instance = false
 		@gtk_window = Gtk::Window.new('Rubyripper')
 		ICONDIR.each{|dir| if File.exist?(file = File.join(dir, 'rubyripper.png')) ; @gtk_window.icon = Gdk::Pixbuf.new(file) ; break end }
-		@gtk_window.set_default_size(520, 400) #width, height
+		@gtk_window.set_default_size(530, 440) #width, height
 		
 		@hbox1 = Gtk::HBox.new(false,5)
 
 		create_buttonbox()
 		create_signals()
 		
-		load_settings()
+		@settingsClass = Settings.new()
+		@settings = @settingsClass.settings
 		welcome_message()
 		
 		@lock = Monitor.new()
+		@updateThread = nil
 		@gtk_window.show_all() #The user interface is up, now load the library
 		scan_drive()
 	end
@@ -93,14 +95,20 @@ attr_reader :change_display, :instances, :update
 		if @buttontext[4].text == _("Exit")
 			savePreferences(); exit()
 		else
-			@rubyripperThread.exit()
-			@rubyripper = nil
-			`killall cdparanoia 2>&1`
-			@buttons.each{|button| button.sensitive = true}
-			change_display(@instances['GtkMetadata'])
+			Thread.new do
+				@rubyripper.cancelRip() # Let rubyripper stop ripping and encoding
+				@rubyripper = nil # kill the instance
+				@rubyripperThread.exit() # kill the thread
+				@buttons.each{|button| button.sensitive = true}
+				change_display(@instances['GtkMetadata'])
+			end
 		end
 	end
 
+	def cancelTocScan
+		`killall cdrdao 2>&1`
+	end
+
 	def savePreferences
 		if @current_instance == 'Preferences'
 			@buttontext[0].set_text('_'+_('Preferences'),true)
@@ -137,26 +145,6 @@ attr_reader :change_display, :instances, :update
 		object.display.show_all()
 	end
 	
-	def load_settings #first set default settings, then override them with custom settings if a config file is found
-		@settings = $rr_defaultSettings
-		
-		if File.exist?(settingsfile = File.join(ENV['HOME'], '.rubyripper/settings'))
-			file = File.new(settingsfile,'r')
-			while line = file.gets
-				key, value = line.split('=', 2)
-				value.rstrip! # remove the trailing newline character
-				if value == "false" ; value = false # replace the string with a bool
-				elsif value == "true" ; value = true  # replace the string with a bool
-				elsif value == "''" ; value = '' #replace a string that contains two quotes with an empty string
-				elsif value.to_i > 0 || value == '0' ; value = value.to_i
-				end
-				if @settings.key?(key) ; @settings[key] = value end
-			end
-			file.close()
-		end
-		if @settings['debug'] ; Thread.abort_on_exception = true end
-	end
-	
 	def welcome_message
 		@instances['ShortMessage'] = ShortMessage.new(@settings['cdrom'])
 		change_display(@instances['ShortMessage'])
@@ -168,7 +156,7 @@ attr_reader :change_display, :instances, :update
 			if @current_instance != 'Preferences'
 				@buttontext[0].set_text('_'+_('Disc info'),true)
 				@buttonicons[0].stock = Gtk::Stock::INFO
-				unless @instances['Preferences'] ; @instances['Preferences'] = Preferences.new(@settings) end
+				unless @instances['Preferences'] ; @instances['Preferences'] = Preferences.new(@settings, @settingsClass) end
 				@instances['Preferences'].display.page = 0
 				change_display(@instances['Preferences'])
 				@buttons[0..2].each{|button| button.sensitive = true} ; if @instances['GtkMetadata'] ; @buttons[3].sensitive = true end ; @buttons[4].sensitive = true
@@ -180,14 +168,34 @@ attr_reader :change_display, :instances, :update
 			end
 		end
 	end
+
+	# give the cdrom drive a few seconds to read the disc
+	def waitForDisc
+		succes = false
+		trial = 1
+
+		while trial < 10
+			@settings['cd'] = Disc.new(@settings, self, 
+				@settings.key?('cd') ? @settings['cd'].freedbString : '')
+			if @settings['cd'].audiotracks != 0
+				succes = true
+				break
+			else
+				puts "No disc found at trial #{trial}!"
+				sleep(1)
+				trial += 1
+			end
+		end
+		return succes
+	end
 	
 	def scan_drive
+		cancelTocScan()
 		@buttons.each{|button| button.sensitive = false}
 		Thread.new do
-			# Analyze audio-cd, don't look at freedb yet. If the current freedb string is the same don't use yaml for metadata
-			@settings['cd'] = Disc.new(@settings['cdrom'], @settings['freedb'], self, @settings['verbose'], @settings.key?('cd') ? @settings['cd'].freedbString : '')
-
-			if @settings['cd'].audiotracks != 0 # if true, a cd is found
+			# Analyze audio-cd, don't look at freedb yet. 
+			#If the current freedb string is the same don't use yaml for metadata
+			if waitForDisc() # if true, a cd is found
 				if @buttontext[2].text != _("Open tray") # We know there's a cd inside so make sure that eject is shown instead of close tray
 					@buttontext[2].set_text('_'+_("Open tray"),true)
 					@buttonicons[2].stock = Gtk::Stock::GOTO_BOTTOM
@@ -245,6 +253,7 @@ attr_reader :change_display, :instances, :update
 					@instances['GtkMetadata'] = false
 					@instances['ShortMessage'].open_tray(@settings['cdrom'])
 					change_display(@instances['ShortMessage'])
+					cancelTocScan()
 					`eject #{@settings['cdrom']}` # spit the cd out
 					@buttontext[2].set_text('_'+_("Close tray"),true)
 					@buttonicons[2].stock = Gtk::Stock::GOTO_TOP
@@ -274,6 +283,7 @@ attr_reader :change_display, :instances, :update
 		@rubyripper = Rubyripper.new(@settings, self) # start a new instance, keep it out the Thread for later callbacks (yet_to_implement)
 		
 		status = @rubyripper.settingsOk
+		puts "status = #{status}" if @settings['debug']
 		if status == true
 			do_rip()
 		else
@@ -285,7 +295,7 @@ attr_reader :change_display, :instances, :update
 	def do_rip
 		@rubyripperThread = Thread.new do
 			@buttons[0..3].each{|button| button.sensitive = false}
- 			if @instances['RipStatus'] == false
+			if @instances['RipStatus'] == false
 				@instances['RipStatus'] = RipStatus.new()
 			else
 				@instances['RipStatus'].reset
@@ -296,16 +306,17 @@ attr_reader :change_display, :instances, :update
 		end
 	end
 	
-	def showSummary
+	def showSummary(succes)
 		@buttons[0..3].each{|button| button.sensitive = true}
-		@instances['Summary'] = Summary.new(@settings['editor'], @settings['filemanager'], @rubyripper.outputDir, @rubyripper.summary)
+		@instances['Summary'] = Summary.new(@settings['editor'], @settings['filemanager'], @rubyripper.outputDir, @rubyripper.summary, succes)
 		change_display(@instances['Summary'])
 		@instances['RipStatus'].reset()
 		@rubyripper = false # some resetting of variables, I suspect some optimization of ruby otherwise would prevent refreshing
 	end
 	
-	def update(modus, value=false) #Threading here is not really necessary, because none of these are cpu intensive.
-		Thread.new do
+	def update(modus, value=false)
+		@updateThread.join if @updateThread != nil # one gui update at a time please
+		@updateThread = Thread.new do
 			if modus == "error"
 				@instances['ShortMessage'].show_message(value)
 				change_display(@instances['ShortMessage'])
@@ -323,7 +334,7 @@ attr_reader :change_display, :instances, :update
 				@instances['DirExists'] = DirExists.new(self, @rubyripper, value)
 				change_display(@instances['DirExists'])
 			elsif modus == "finished"
-				showSummary()
+				showSummary(value)
 				if @settings['eject'] == true
 					@buttontext[2].text =_("Close tray")
 					@buttonicons[2].stock = Gtk::Stock::GOTO_TOP
@@ -350,7 +361,7 @@ attr_reader :change_display, :instances, :update
 end
 
 class ShortMessage
-attr_reader :display, :no_disc_found, :open_tray, :close_tray, :ask_for_disc, :no_eject_found, :show_message
+attr_reader :display
 
 	def initialize(cdrom)
 		@display = Gtk::Label.new(_("Welcome to rubyripper %s.\nScanning drive %s for an audio disc...") % [$rr_version, cdrom])
@@ -382,7 +393,7 @@ attr_reader :display, :no_disc_found, :open_tray, :close_tray, :ask_for_disc, :n
 end
 
 class GtkMetadata
-attr_reader :refreshDisc, :updateMetadata, :display, :save_updates, :tracks_to_rip
+attr_reader :display, :tracks_to_rip
 
 	def initialize(cdinfo)
 		@cd = cdinfo
@@ -498,7 +509,7 @@ attr_reader :refreshDisc, :updateMetadata, :display, :save_updates, :tracks_to_r
 			@checkTrackArray << Gtk::CheckButton.new((track + 1).to_s)
 			@varArtistEntryArray << Gtk::Entry.new()
 			@trackEntryArray << Gtk::Entry.new()
-			@lengthLabelArray << Gtk::Label.new(@cd.lengthText[track])
+			@lengthLabelArray << Gtk::Label.new(@cd.getLengthText(track + 1))
 		end
 	end
 
@@ -591,7 +602,9 @@ attr_reader :refreshDisc, :updateMetadata, :display, :save_updates, :tracks_to_r
 		else
 			@discNumberSpin.value += 1.0
 		end
-		if @cd.md.varArtists.empty? : @varCheckbox.active = false end
+		if @cd.md.varArtists.empty?
+			@varCheckbox.active = false
+		end
 	end
 
 	def updateTracks()
@@ -633,7 +646,7 @@ attr_reader :refreshDisc, :updateMetadata, :display, :save_updates, :tracks_to_r
 
 	def unsetVarArtist()
 		# giving the backend the signal to revert last actions
-		@cd.md.undoVarArtist()
+		@cd.md.undoVarArtist() if !@cd.md.varArtists.empty?
 		# reset the Trackname fields (give full trackname, including detected artists)
 		@cd.audiotracks.times{|index| @trackEntryArray[index].text = @cd.md.tracklist[index]}
 		# remove all current objects from array, as we're repacking them
@@ -654,7 +667,7 @@ attr_reader :refreshDisc, :updateMetadata, :display, :save_updates, :tracks_to_r
 		@tracks_to_rip = Array.new #reset the array
 		
 		if image
-		    @tracks_to_rip = [0]
+		    @tracks_to_rip = ["image"]
 		else
 		    @cd.audiotracks.times do |index|
 			@cd.md.tracklist[index] = @trackEntryArray[index].text
@@ -669,14 +682,19 @@ attr_reader :refreshDisc, :updateMetadata, :display, :save_updates, :tracks_to_r
 end
 
 class Preferences
-attr_reader :display, :save
+attr_reader :display
 
-	def initialize(settings)
+	def initialize(settings, settingsClass)
 		@settings = settings
+		@settingsClass = settingsClass
 		@display = Gtk::Notebook.new # Create a notebook (multiple pages)
 		ripobjects_frame1()
 		ripobjects_frame2()
 		ripobjects_frame3()
+		gapObjectsFrame1()
+		gapObjectsFrame2()
+		gapObjectsFrame3()
+		gapObjectsFrame4()
 		codecobjects_frame1()
 		codecobjects_frame2()
 		codecobjects_frame3()
@@ -698,8 +716,15 @@ attr_reader :display, :save
 		@rip_entry.text = @settings['rippersettings']
 		@eject.active = @settings['eject']
 		@no_log.active = @settings['no_log']
+#toc settings
 		@create_cue.active = @settings['create_cue']
 		@create_single_file.active = @settings['image']
+		@ripHiddenAudio.active = @settings['ripHiddenAudio']
+		@markHiddenTrackSpin.value = @settings['minLengthHiddenTrack'].to_f
+		@appendPregaps.active = @settings['pregaps'] == 'append'
+		@prependPregaps.active = @settings['pregaps'] == 'prepend'
+		@correctPreEmphasis.active = @settings['preEmphasis'] == 'sox'
+		@doNotCorrectPreEmphasis.active = @settings['preEmphasis'] == 'cue'
 #codec settings
 		@flac.active = @settings['flac']
 		@vorbis.active = @settings['vorbis']
@@ -743,8 +768,13 @@ attr_reader :display, :save
 		@settings['rippersettings'] = @rip_entry.text
 		@settings['eject'] = @eject.active?
 		@settings['no_log'] = @no_log.active?
+#toc settings
 		@settings['create_cue'] = @create_cue.active?
 		@settings['image'] = @create_single_file.active?
+		@settings['ripHiddenAudio'] = @ripHiddenAudio.active?
+		@settings['minLengthHiddenTrack'] = @markHiddenTrackSpin.value.to_i
+		@settings['pregaps'] = @appendPregaps.active? ? 'append' : 'prepend'
+		@settings['preEmphasis'] = @correctPreEmphasis.active? ? 'sox' : 'cue'
 #codec settings
 		@settings['flac'] = @flac.active?
 		@settings['vorbis'] = @vorbis.active?
@@ -759,6 +789,15 @@ attr_reader :display, :save
 		@settings['noSpaces'] = @noSpaces.active?
 		@settings['noCapitals'] = @noCapitals.active?
 		@settings['maxThreads'] = @maxThreads.value.to_i
+
+		# The gtk2 interface gets crazy on older versions of the bindings and threads
+		if Gtk::BINDING_VERSION[1] < 18 && @settings['maxThreads'] > 0
+			@settings['maxThreads'] = 0
+			puts "WARNING: Threads are not supported on ruby gtk2-bindings"
+			puts "that are older than 0.18.0. Setting them to zero."
+			puts "Please upgrade your bindings if you want threads."
+		end
+		
 		@settings['normalize'] = if @normalize.active == 0 ; false elsif @normalize.active == 1 ; "replaygain" else "normalize" end
 		@settings['gain'] = if @modus.active ==0 ; "album" else "track" end
 #freedb
@@ -776,19 +815,7 @@ attr_reader :display, :save
 		@settings['debug'] = @debug.active?
 		@settings['editor'] = @editor_entry.text
 		@settings['filemanager'] = @filemanager_entry.text
-		if @settings['debug'] ; Thread.abort_on_exception = true end
-		save_configfile() #also update the config file
-	end
-	
-	def save_configfile
-		if not File.directory?(dirname = File.join(ENV['HOME'], '.rubyripper'))
-			Dir.mkdir(dirname)
-		end
-		file = File.new(File.join(ENV['HOME'], '.rubyripper/settings'), 'w')
-		@settings.each do |key, value|
-			file.puts "#{key}=#{value}" if $rr_defaultSettings.include?(key)
-		end
-		file.close()
+		@settingsClass.save(@settings) #also update the config file
 	end
 	
 #Today is a great day to start counting with 40 :) Actually I worked backwards and needed to make sure I had enough room in the beginning.
@@ -811,7 +838,7 @@ attr_reader :display, :save
 		@table40.attach(@cdrom_offset_spin, 1, 2, 1, 2, Gtk::FILL, Gtk::SHRINK, 0, 0)
 		@table40.attach(@offset_button, 2, 3, 1, 2, Gtk::FILL, Gtk::SHRINK, 0, 0)
 #connect signal
-		@offset_button.signal_connect("clicked") {Thread.new{`#{browser()} #{@offset_button.uri}`}}
+		@offset_button.signal_connect("clicked") {Thread.new{`#{@settings['browser']} #{@offset_button.uri}`}}
 #create frame
 		@frame40 = Gtk::Frame.new(_('Cdrom device'))
 		@frame40.set_shadow_type(Gtk::SHADOW_ETCHED_IN)
@@ -854,7 +881,7 @@ attr_reader :display, :save
 	end
 	
 	def ripobjects_frame3 #Ripping related frame
-		@table60 = Gtk::Table.new(4,3,false)
+		@table60 = Gtk::Table.new(2,3,false)
 		@table60.column_spacings = 5
 		@table60.row_spacings = 4
 		@table60.border_width = 7
@@ -862,16 +889,12 @@ attr_reader :display, :save
 		@rip_label = Gtk::Label.new(_("Pass cdparanoia options:")) ; @rip_label.set_alignment(0.0, 0.5)
 		@eject= Gtk::CheckButton.new(_('Eject cd when finished'))
 		@no_log = Gtk::CheckButton.new(_('Only keep logfile if correction is needed'))
-		@create_cue = Gtk::CheckButton.new(_('Create cuesheet'))
-		@create_single_file = Gtk::CheckButton.new(_('Rip CD to single file'))
 		@rip_entry= Gtk::Entry.new ; @rip_entry.width_request = 120
 #pack objects
 		@table60.attach(@rip_label, 0, 1, 0, 1, Gtk::FILL, Gtk::SHRINK, 0, 0)
-		@table60.attach(@eject, 0, 2, 1, 2, Gtk::FILL, Gtk::SHRINK, 0, 0)
 		@table60.attach(@rip_entry, 1, 2, 0, 1, Gtk::SHRINK, Gtk::SHRINK, 0, 0)
+		@table60.attach(@eject, 0, 2, 1, 2, Gtk::FILL, Gtk::SHRINK, 0, 0)
 		@table60.attach(@no_log, 0, 2, 2, 3, Gtk::FILL|Gtk::SHRINK, Gtk::SHRINK, 0, 0)
-		@table60.attach(@create_cue, 0, 2, 3, 4, Gtk::FILL|Gtk::SHRINK, Gtk::SHRINK, 0, 0)
-		@table60.attach(@create_single_file, 0, 2, 4, 5, Gtk::FILL|Gtk::SHRINK, Gtk::SHRINK, 0, 0)
 #create frame
 		@frame60 = Gtk::Frame.new(_('Ripping related'))
 		@frame60.set_shadow_type(Gtk::SHADOW_ETCHED_IN)
@@ -883,7 +906,143 @@ attr_reader :display, :save
 		@page1_label = Gtk::Label.new(_("Secure Ripping"))
 		@display.append_page(@page1, @page1_label)
 	end
+
+	def gapObjectsFrame1
+		@tableToc1 = Gtk::Table.new(3,3,false)
+		@tableToc1.column_spacings = 5
+		@tableToc1.row_spacings = 4
+		@tableToc1.border_width = 7
+#create objects
+		@ripHiddenAudio = Gtk::CheckButton.new(_('Rip hidden audio sectors'))
+		@markHiddenTrackLabel1 = Gtk::Label.new(_('Mark as a hidden track when bigger than'))
+		@markHiddenTrackLabel2 = Gtk::Label.new(_('seconds'))
+		@markHiddenTrackSpin = Gtk::SpinButton.new(0, 30, 1)
+		@markHiddenTrackSpin.value = 2.0
+		@ripHiddenAudio.tooltip_text = _("Uncheck this if cdparanoia crashes with your ripping drive.")
+		text = _("A hidden track will rip to a seperate file if used in track modus.\nIf it's smaller the sectors will be prepended to the first track.")
+		@markHiddenTrackSpin.tooltip_text = text
+		@markHiddenTrackLabel1.tooltip_text = text
+		@markHiddenTrackLabel2.tooltip_text = text
+#pack objects
+		@tableToc1.attach(@ripHiddenAudio, 0, 1, 0, 1, Gtk::FILL, Gtk::SHRINK, 0, 0)
+		@tableToc1.attach(@markHiddenTrackLabel1, 0, 1, 1, 2, Gtk::FILL, Gtk::SHRINK, 0, 0)
+		@tableToc1.attach(@markHiddenTrackSpin, 1, 2, 1, 2, Gtk::FILL, Gtk::SHRINK, 0, 0)
+		@tableToc1.attach(@markHiddenTrackLabel2, 2, 3, 1, 2, Gtk::FILL, Gtk::SHRINK, 0, 0)
+#create frame
+		@ripHiddenAudio.signal_connect("clicked"){@markHiddenTrackSpin.sensitive = @ripHiddenAudio.active?}
+		@frameToc1 = Gtk::Frame.new(_('Audio sectors before track 1'))
+		@frameToc1.set_shadow_type(Gtk::SHADOW_ETCHED_IN)
+		@frameToc1.border_width = 5
+		@frameToc1.add(@tableToc1)
+	end
 	
+	def gapObjectsFrame2
+		@tableToc2 = Gtk::Table.new(3,2,false)
+		@tableToc2.column_spacings = 5
+		@tableToc2.row_spacings = 4
+		@tableToc2.border_width = 7
+		#create objects
+		@create_cue = Gtk::CheckButton.new(_('Create cuesheet'))
+		@create_single_file = Gtk::CheckButton.new(_('Rip CD to single file'))
+#pack objects
+		@tableToc2.attach(@create_cue, 0, 2, 1, 2, Gtk::FILL, Gtk::SHRINK, 0, 0)
+		@tableToc2.attach(@create_single_file, 0, 2, 2, 3, Gtk::FILL|Gtk::SHRINK, Gtk::SHRINK, 0, 0)
+#create frame
+		@frameToc2 = Gtk::Frame.new(_('Advanced Toc analysis'))
+		@frameToc2.set_shadow_type(Gtk::SHADOW_ETCHED_IN)
+		@frameToc2.border_width = 5
+		@vboxToc = Gtk::VBox.new()
+		@vboxToc.pack_start(@tableToc2,false,false)
+		@frameToc2.add(@vboxToc)
+# build hbox for cdrdao
+		@cdrdaoHbox = Gtk::HBox.new(false, 5)
+		@cdrdao = Gtk::Label.new(_('Cdrdao installed?'))
+		@cdrdaoImage = Gtk::Image.new(Gtk::Stock::CANCEL, Gtk::IconSize::BUTTON)
+		@cdrdaoHbox.pack_start(@cdrdao, false, false, 5)
+		@cdrdaoHbox.pack_start(@cdrdaoImage, false, false)
+	end
+	
+	def gapObjectsFrame3
+		@tableToc3 = Gtk::Table.new(3,3,false)
+		@tableToc3.column_spacings = 5
+		@tableToc3.row_spacings = 4
+		@tableToc3.border_width = 7
+#create objects
+		@appendPregaps = Gtk::RadioButton.new(_('Append pregap to the previous track'))
+		@prependPregaps = Gtk::RadioButton.new(@appendPregaps, _('Prepend pregaps to the track'))
+#pack objects
+		@tableToc3.attach(@appendPregaps, 0, 1, 0, 1, Gtk::FILL, Gtk::SHRINK, 0, 0)
+		@tableToc3.attach(@prependPregaps, 0, 1, 1, 2, Gtk::FILL, Gtk::SHRINK, 0, 0)
+#create frame
+		@frameToc3 = Gtk::Frame.new(_('Handling pregaps other than track 1'))
+		@frameToc3.set_shadow_type(Gtk::SHADOW_ETCHED_IN)
+		@frameToc3.border_width = 5
+		@frameToc3.add(@tableToc3)
+		@vboxToc.pack_start(@frameToc3,false,false)
+	end
+
+	def gapObjectsFrame4
+		@tableToc4 = Gtk::Table.new(3,3,false)
+		@tableToc4.column_spacings = 5
+		@tableToc4.row_spacings = 4
+		@tableToc4.border_width = 7
+#create objects
+		@correctPreEmphasis = Gtk::RadioButton.new(_('Correct pre-emphasis tracks with sox'))
+		@doNotCorrectPreEmphasis = Gtk::RadioButton.new(@correctPreEmphasis, _("Save the pre-emphasis tag in the cuesheet."))
+#pack objects
+		@tableToc4.attach(@correctPreEmphasis, 0, 1, 0, 1, Gtk::FILL, Gtk::SHRINK, 0, 0)
+		@tableToc4.attach(@doNotCorrectPreEmphasis, 0, 1, 1, 2, Gtk::FILL, Gtk::SHRINK, 0, 0)
+#create frame
+		@frameToc4 = Gtk::Frame.new(_('Handling tracks with pre-emphasis'))
+		@frameToc4.set_shadow_type(Gtk::SHADOW_ETCHED_IN)
+		@frameToc4.border_width = 5
+		@frameToc4.add(@tableToc4)
+		@vboxToc.pack_start(@frameToc4,false,false)
+#pack all frames into a single page
+		setSignalsToc()
+		@pageToc = Gtk::VBox.new #One VBox to rule them all
+		[@frameToc1, @cdrdaoHbox, @frameToc2].each{|frame| @pageToc.pack_start(frame,false,false)}
+		@pageTocLabel = Gtk::Label.new(_("TOC analysis"))
+		@display.append_page(@pageToc, @pageTocLabel)
+	end
+	
+	#check if cdrdao is installed
+	def cdrdaoInstalled
+		if installed('cdrdao')
+			@cdrdaoImage.stock = Gtk::Stock::APPLY
+			@frameToc2.each{|child| child.sensitive = true}
+		else
+			@cdrdaoImage.stock = Gtk::Stock::CANCEL
+			@create_cue.active = false
+			@frameToc2.each{|child| child.sensitive = false}
+		end
+	end
+	
+	# signal for createCue
+	def createCue
+		@create_single_file.sensitive = @create_cue.active?
+		@create_single_file.active = false if !@create_cue.active?
+		@tableToc3.each{|child| child.sensitive = @create_cue.active?}
+		@tableToc4.each{|child| child.sensitive = @create_cue.active?}
+	end
+	
+	# signal for create single file
+	def createSingle
+		@tableToc3.each{|child| child.sensitive = !@create_single_file.active?}
+		@correctPreEmphasis.active = true
+		@doNotCorrectPreEmphasis.sensitive = !@create_single_file.active?
+	end
+	
+	#set signals for the toc
+	def setSignalsToc
+		cdrdaoInstalled()
+		createSingle()
+		createCue()
+		@create_cue.signal_connect("clicked"){createCue()}
+		@create_cue.signal_connect("clicked"){`killall cdrdao 2>1` if !@create_cue.active?}
+		@create_single_file.signal_connect("clicked"){createSingle()}
+	end
+
 	def codecobjects_frame1 # Select audio codecs frame
 		@table70 = Gtk::Table.new(6,2,false)
 		@table70.column_spacings = 5
@@ -1046,10 +1205,13 @@ attr_reader :display, :save
 		@basedir_entry.signal_connect("button_release_event"){@example_label.text = get_example_filename_normal(@basedir_entry.text, @naming_normal_entry.text) ; false}
 		@naming_normal_entry.signal_connect("key_release_event"){@example_label.text = get_example_filename_normal(@basedir_entry.text, @naming_normal_entry.text) ; false}
 		@naming_normal_entry.signal_connect("button_release_event"){@example_label.text = get_example_filename_normal(@basedir_entry.text, @naming_normal_entry.text) ; false}
+		@naming_normal_entry.signal_connect("focus-out-event"){if not File.dirname(@naming_normal_entry.text) =~ /%a|%b/ ; @naming_normal_entry.text = "%a (%y) %b/" + @naming_normal_entry.text; preventStupidness() end; false}
 		@naming_various_entry.signal_connect("key_release_event"){@example_label.text = get_example_filename_various(@basedir_entry.text, @naming_various_entry.text) ; false}
 		@naming_various_entry.signal_connect("button_release_event"){@example_label.text = get_example_filename_various(@basedir_entry.text, @naming_various_entry.text) ; false}
+		@naming_various_entry.signal_connect("focus-out-event"){if not File.dirname(@naming_various_entry.text) =~ /%a|%b/ ; @naming_various_entry.text = "%a (%y) %b/" + @naming_various_entry.text; preventStupidness() end; false}
 		@naming_image_entry.signal_connect("key_release_event"){@example_label.text = get_example_filename_various(@basedir_entry.text, @naming_image_entry.text) ; false}
 		@naming_image_entry.signal_connect("button_release_event"){@example_label.text = get_example_filename_various(@basedir_entry.text, @naming_image_entry.text) ; false}
+		@naming_image_entry.signal_connect("focus-out-event"){if not File.dirname(@naming_image_entry.text) =~ /%a|%b/ ; @naming_image_entry.text = "%a (%y) %b/" + @naming_image_entry.text; preventStupidness() end; false}
 #packing 2nd column
 		@table100.attach(@basedir_entry, 1, 2, 0, 1, Gtk::EXPAND|Gtk::FILL, Gtk::SHRINK, 0, 0)
 		@table100.attach(@naming_normal_entry, 1, 2, 1, 2, Gtk::EXPAND|Gtk::FILL, Gtk::SHRINK, 0, 0)
@@ -1062,6 +1224,13 @@ attr_reader :display, :save
 		@frame100.add(@table100)
 	end
 
+	# Would you believe this actually prevents bug reports?
+	def preventStupidness()
+		puts "You need to make a subdirectory with at least the artist or album"
+		puts "name in it. Otherwise your directory will be overwritten each time!"
+		puts "To protect you from making these unwise choices this is corrected :P"
+	end
+
 #Small table needed for setting programs
 #log file viewer 	| entry
 #file manager 	| entry
@@ -1110,13 +1279,19 @@ attr_reader :display, :save
 		@page4 = Gtk::VBox.new()
 		[@frame100, @frame110, @frame120].each{|frame| @page4.pack_start(frame,false,false)}
 		@page4_label = Gtk::Label.new(_("Other"))
-		@display.signal_connect("switch_page"){|a, b, page| if page == 3; @example_label.text = get_example_filename_normal(@basedir_entry.text, @naming_normal_entry.text) end}
+		@display.signal_connect("switch_page") do |a, b, page|
+			if page == 1
+				cdrdaoInstalled()
+			elsif page == 4
+				@example_label.text = get_example_filename_normal(@basedir_entry.text, @naming_normal_entry.text)
+			end
+		end
 		@display.append_page(@page4, @page4_label)
 	end
 end
 
 class RipStatus
-attr_reader :textview, :rip_bar, :enc_bar, :display, :reset
+attr_reader :textview, :rip_bar, :enc_bar, :display
 
 	def initialize
 		create_status_objects()
@@ -1171,7 +1346,7 @@ class DirExists
 attr_reader :display
 
 	def initialize(gui, rubyripper, dirname)
-		@label = Gtk::Label.new(_("The directory %s\nalready exist. What do you want rubyripper to do?") % [dirname])
+		@label = Gtk::Label.new(_("The directory %s already exists.\n\nWhat do you want rubyripper to do?") % [dirname])
 		@label.wrap = true
 		@image = Gtk::Image.new(Gtk::Stock::DIALOG_QUESTION, Gtk::IconSize::DIALOG)
 		
@@ -1180,7 +1355,7 @@ attr_reader :display
 		@separator = Gtk::HSeparator.new
 
 		@buttons = [Gtk::Button.new, Gtk::Button.new, Gtk::Button.new]
-		@labels = [Gtk::Label.new(_("Cancel rip")), Gtk::Label.new(_("Overwrite\ndirectory")), Gtk::Label.new(_("Auto rename\ndirectory"))]
+		@labels = [Gtk::Label.new(_("Cancel rip")), Gtk::Label.new(_("Delete existing\ndirectory")), Gtk::Label.new(_("Auto rename\ndirectory"))]
 		@images = [Gtk::Image.new(Gtk::Stock::CANCEL, Gtk::IconSize::LARGE_TOOLBAR), Gtk::Image.new(Gtk::Stock::CLEAR, Gtk::IconSize::LARGE_TOOLBAR), Gtk::Image.new(Gtk::Stock::OK, Gtk::IconSize::LARGE_TOOLBAR)]
 		@hboxes = [Gtk::HBox.new, Gtk::HBox.new, Gtk::HBox.new]
 		@buttonbox = Gtk::HBox.new
@@ -1251,9 +1426,14 @@ end
 class Summary
 attr_reader :display
 
-	def initialize(editor, filemanager, directory, summary)
-		@label1 = Gtk::Label.new(_("Rubyripper is finished with ripping the cd.\nA short summary is shown below."))
-		@image1 = Gtk::Image.new(Gtk::Stock::DIALOG_INFO, Gtk::IconSize::DIALOG)
+	def initialize(editor, filemanager, directory, summary, succes)
+		if succes == true
+			@label1 = Gtk::Label.new(_("The rip has succesfully finished.\nA short summary is shown below."))
+			@image1 = Gtk::Image.new(Gtk::Stock::DIALOG_INFO, Gtk::IconSize::DIALOG)
+		else
+			@label1 = Gtk::Label.new(_("The rip had some problems.\nA short summary is shown below."))
+			@image1 = Gtk::Image.new(Gtk::Stock::DIALOG_ERROR, Gtk::IconSize::DIALOG)
+		end
 		@hbox1 = Gtk::HBox.new()
 		[@image1, @label1].each{|object| @hbox1.pack_start(object)}
 		@hbox1.border_width = 10
